# Data Science for Business II | Week 2 Example 2: Distribution system design
# Author: Lauri Neuvonen

# SETUP
# We'll use the Gurobi optimizer in this exercise, so we'll import it and Pandas first:

import pandas as pd
import numpy as np
from gurobipy import *

# For this example, the data has been saved in separate text files, with ";" as column delimiter
# Let's import the data into dataframes by using Pandas read_csv function

plantOptions = pd.read_csv("Ex2-plant_options.txt", delimiter=";", index_col = 0)
distributionCenters = pd.read_csv("Ex2-distribution_centers.txt", delimiter=";", index_col = 0)
shippingCosts = pd.read_csv("Ex2-shipping_costs.txt", delimiter = ";", index_col = 0)

N_plants = len(shippingCosts.index)
N_distrCs = len(distributionCenters.index)

print(shippingCosts)
try:

    m = Model("Example 2 optimizer") # creates an empty model

    x = m.addVars(N_plants, N_distrCs, name="routeShipping") # adds variables depicting transportation volumes from plants to distribution centers
    y = m.addVars(N_plants-1, vtype=GRB.BINARY, name="newPlant") # variables indicating if a new plant is built

    # Annual capacity constraints. These are inserted using a for loop that loops through the first 4 capacity constraints:
    m.addConstrs(x[i,0] + x[i,1] + x[i,2] - plantOptions[" Annual Capacity "][i]*y[i] <= 0 for i in range(N_plants-1))

    # The last capacity constraint for the existing plant
    m.addConstr(x[N_plants-1, 0] + x[N_plants-1, 1] + x[N_plants-1, 2] <= 30000)

    # Demand constraints, again inserted using a for loop:
    m.addConstrs(x[0, j] + x[1, j] + x[2, j] + x[3, j] + x[4, j] == distributionCenters[" Annual demand"][j] for j in range(N_distrCs))

    # Plant in either Dallas OR Fort Worth (uncomment to activate):
    # m.addConstr(y[0] + y[1] == 1)

    # Get shipping costs for the objective function
    SC = shippingCosts.values
    print("SC: ", SC)
    totalShippingCosts = np.concatenate(SC).dot(x.values())

    # Set objective function and select to minimize:
    m.setObjective(sum(plantOptions[" Annual Fixed Cost"]*y.values()) + totalShippingCosts, GRB.MINIMIZE)

    # Optimize, i.e. solve the model
    m.optimize()

    # This prints the model description to an .lp file (could be used e.g. to export the model)
    m.write('Ex2model.lp')

    # get results into var_values
    var_values = m.getVars()

    # Just print the results:
    for v in var_values:
        print('%s %g' % (v.varName, v.x))

    print('Obj: %g' % m.objVal)

except GurobiError as e:
    print('Error code ' + str(e.errno) + ": " + str(e))

except AttributeError:
    print('Encountered an attribute error')


